@@ -9,13 +9,14 @@ from urllib.parse import urlparse
 
 import click
 
+from . import __codename__
 from . import __version__
-from .controllers import Cache
 from .controllers import CastState
-from .controllers import get_chromecasts
 from .controllers import setup_cast
 from .controllers import StateFileError
 from .controllers import StateMode
+from .discovery import cast_device_ip_exists
+from .discovery import get_cast_devices_info
 from .error import CastError
 from .error import CattUserError
 from .error import CliError
@@ -31,6 +32,8 @@ CONFIG_DIR = Path(click.get_app_dir("catt"))
 CONFIG_PATH = Path(CONFIG_DIR, "catt.cfg")
 STATE_PATH = Path(CONFIG_DIR, "state.json")
 
+WAIT_PLAY_TIMEOUT = 30
+
 
 class CattTimeParamType(click.ParamType):
     def convert(self, value, param, ctx):
@@ -62,7 +65,7 @@ class YtdlOptParamType(click.ParamType):
 YTDL_OPT = YtdlOptParamType()
 
 
-def process_url(ctx, param, value):
+def process_url(ctx, param, value: str):
     if value == "-":
         stdin_text = click.get_text_stream("stdin")
         if not stdin_text.isatty():
@@ -95,18 +98,21 @@ def process_subtitles(ctx, param, value):
     return value
 
 
-def process_device(ctx, param, value):
+def process_device(device_desc, aliases):
     """
     Resolve real device name when value is an alias.
 
-    :param value: Can be an ip-address or a name (alias or real name).
-    :type value: str
+    :param device_desc: Can be an ip-address or a name (alias or real name).
+    :type device_desc: str
+    :param aliases: Dictionary of device aliases and their corresponding real names.
+    :type aliases: Dict[str, str]
     """
-
-    if is_ipaddress(value):
-        return value
+    if is_ipaddress(device_desc):
+        return device_desc
     else:
-        return ctx.default_map["aliases"].get(value, value)
+        if device_desc:
+            device_desc = device_desc.lower()
+        return aliases.get(device_desc, device_desc)
 
 
 def fail_if_no_ip(ipaddr):
@@ -125,30 +131,52 @@ CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
 
 
 @click.group(context_settings=CONTEXT_SETTINGS)
-@click.option("--delete-cache", is_flag=True, help="Empty the Chromecast discovery cache.")
-@click.option("-d", "--device", metavar="NAME_OR_IP", callback=process_device, help="Select Chromecast device.")
-@click.version_option(version=__version__, prog_name="catt", message="%(prog)s v%(version)s, Yearning Yachtman.")
+@click.option("-d", "--device", metavar="NAME_OR_IP", help="Select Chromecast device.")
+@click.version_option(
+    version=__version__,
+    prog_name="catt",
+    message="%(prog)s v%(version)s, " + __codename__ + ".",
+)
 @click.pass_context
-def cli(ctx, delete_cache, device):
-    if delete_cache:
-        Cache().clear()
-    ctx.obj["device"] = device
+def cli(ctx, device):
+    device_from_config = ctx.obj["options"].get("device")
+    ctx.obj["selected_device"] = process_device(device or device_from_config, ctx.obj["aliases"])
+    ctx.obj["selected_device_is_from_cli"] = bool(device)
 
 
 @cli.command(short_help="Send a video to a Chromecast for playing.")
 @click.argument("video_url", callback=process_url)
-@click.option("-s", "--subtitles", callback=process_subtitles, metavar="SUB", help="Specify a subtitles file.")
+@click.option(
+    "-s",
+    "--subtitles",
+    callback=process_subtitles,
+    metavar="SUB",
+    help="Specify a subtitles file.",
+)
 @click.option(
     "-f",
     "--force-default",
     is_flag=True,
     help="Force use of the default Chromecast app (use if a custom app doesn't work).",
 )
-@click.option("-r", "--random-play", is_flag=True, help="Play random item from playlist, if applicable.")
 @click.option(
-    "--no-subs", is_flag=True, default=False, help="Don't try to load subtitles automatically from the local folder."
+    "-r",
+    "--random-play",
+    is_flag=True,
+    help="Play random item from playlist, if applicable.",
+)
+@click.option(
+    "--no-subs",
+    is_flag=True,
+    default=False,
+    help="Don't try to load subtitles automatically from the local folder.",
+)
+@click.option(
+    "-n",
+    "--no-playlist",
+    is_flag=True,
+    help="Play only video, if url contains both video and playlist ids.",
 )
-@click.option("-n", "--no-playlist", is_flag=True, help="Play only video, if url contains both video and playlist ids.")
 @click.option(
     "-y",
     "--ytdl-option",
@@ -158,6 +186,13 @@ def cli(ctx, delete_cache, device):
     help="YouTube-DL option. "
     "Should be passed as `-y option=value`, and can be specified multiple times (implies --force-default).",
 )
+@click.option(
+    "-t",
+    "--seek-to",
+    type=CATT_TIME,
+    metavar="TIME",
+    help="Start playback at specific timestamp.",
+)
 @click.option(
     "-b",
     "--block",
@@ -167,12 +202,27 @@ def cli(ctx, delete_cache, device):
     "Currently exits after playback of single media, so not useful with playlists yet.",
 )
 @click.pass_obj
-def cast(settings, video_url, subtitles, force_default, random_play, no_subs, no_playlist, ytdl_option, block=False):
+def cast(
+    settings,
+    video_url: str,
+    subtitles,
+    force_default: bool,
+    random_play: bool,
+    no_subs: bool,
+    no_playlist: bool,
+    ytdl_option,
+    seek_to: str,
+    block: bool = False,
+):
     controller = "default" if force_default or ytdl_option else None
     playlist_playback = False
     st_thr = su_thr = subs = None
     cst, stream = setup_cast(
-        settings["device"], video_url=video_url, prep="app", controller=controller, ytdl_options=ytdl_option
+        settings["selected_device"],
+        video_url=video_url,
+        prep="app",
+        controller=controller,
+        ytdl_options=ytdl_option,
     )
     media_is_image = stream.guessed_content_category == "image"
     local_or_remote = "local" if stream.is_local_file else "remote"
@@ -180,7 +230,11 @@ def cast(settings, video_url, subtitles, force_default, random_play, no_subs, no
     if stream.is_local_file:
         fail_if_no_ip(stream.local_ip)
         st_thr = create_server_thread(
-            video_url, stream.local_ip, stream.port, stream.guessed_content_type, single_req=media_is_image
+            video_url,
+            stream.local_ip,
+            stream.port,
+            stream.guessed_content_type,
+            single_req=media_is_image,
         )
     elif stream.is_playlist and not (no_playlist and stream.video_id):
         if stream.playlist_length == 0:
@@ -207,12 +261,20 @@ def cast(settings, video_url, subtitles, force_default, random_play, no_subs, no
             fail_if_no_ip(stream.local_ip)
             subs = SubsInfo(subtitles, stream.local_ip, stream.port + 1)
             su_thr = create_server_thread(
-                subs.file, subs.local_ip, subs.port, "text/vtt;charset=utf-8", single_req=True
+                subs.file,
+                subs.local_ip,
+                subs.port,
+                "text/vtt;charset=utf-8",
+                single_req=True,
             )
 
         click.echo("Casting {} file {}...".format(local_or_remote, video_url))
         click.echo(
-            '{} "{}" on "{}"...'.format("Showing" if media_is_image else "Playing", stream.video_title, cst.cc_name)
+            '{} "{}" on "{}"...'.format(
+                "Showing" if media_is_image else "Playing",
+                stream.video_title,
+                cst.cc_name,
+            )
         )
         if cst.info_type == "url":
             cst.play_media_url(
@@ -221,16 +283,17 @@ def cast(settings, video_url, subtitles, force_default, random_play, no_subs, no
                 content_type=stream.guessed_content_type,
                 subtitles=subs.url if subs else None,
                 thumb=stream.video_thumbnail,
+                current_time=seek_to,
             )
         elif cst.info_type == "id":
-            cst.play_media_id(stream.video_id)
+            cst.play_media_id(stream.video_id, current_time=seek_to)
         else:
             raise ValueError("Invalid or undefined info type")
 
     if stream.is_local_file or subs:
         click.echo("Serving local file(s).")
     if not media_is_image and (stream.is_local_file or block):
-        if not cst.wait_for(["PLAYING"], timeout=10):
+        if not cst.wait_for(["PLAYING"], timeout=WAIT_PLAY_TIMEOUT):
             raise CliError("Playback of {} file has failed".format(local_or_remote))
         cst.wait_for(["UNKNOWN", "IDLE"])
     elif (stream.is_local_file and media_is_image) or subs:
@@ -242,17 +305,27 @@ def cast(settings, video_url, subtitles, force_default, random_play, no_subs, no
 @click.argument("url", callback=process_url)
 @click.pass_obj
 def cast_site(settings, url):
-    cst = setup_cast(settings["device"], controller="dashcast", action="load_url", prep="app")
+    cst = setup_cast(
+        settings["selected_device"],
+        controller="dashcast",
+        action="load_url",
+        prep="app",
+    )
     click.echo('Casting {} on "{}"...'.format(url, cst.cc_name))
     cst.load_url(url)
 
 
 @cli.command(short_help="Add a video to the queue (YouTube only).")
 @click.argument("video_url", callback=process_url)
-@click.option("-n", "--play-next", is_flag=True, help="Add video immediately after currently playing video.")
+@click.option(
+    "-n",
+    "--play-next",
+    is_flag=True,
+    help="Add video immediately after currently playing video.",
+)
 @click.pass_obj
 def add(settings, video_url, play_next):
-    cst, stream = setup_cast(settings["device"], video_url=video_url, action="add", prep="control")
+    cst, stream = setup_cast(settings["selected_device"], video_url=video_url, action="add", prep="control")
     if cst.name != stream.extractor or not (stream.is_remote_file or stream.is_playlist_with_active_entry):
         raise CliError("This url cannot be added to the queue")
     click.echo('Adding video id "{}" to the queue.'.format(stream.video_id))
@@ -266,7 +339,12 @@ def add(settings, video_url, play_next):
 @click.argument("video_url", callback=process_url)
 @click.pass_obj
 def remove(settings, video_url):
-    cst, stream = setup_cast(settings["device"], video_url=video_url, action="remove", prep="control")
+    cst, stream = setup_cast(
+        settings["selected_device"],
+        video_url=video_url,
+        action="remove",
+        prep="control",
+    )
     if cst.name != stream.extractor or not stream.is_remote_file:
         raise CliError("This url cannot be removed from the queue")
     click.echo('Removing video id "{}" from the queue.'.format(stream.video_id))
@@ -276,28 +354,28 @@ def remove(settings, video_url):
 @cli.command(short_help="Clear the queue (YouTube only).")
 @click.pass_obj
 def clear(settings):
-    cst = setup_cast(settings["device"], action="clear", prep="control")
+    cst = setup_cast(settings["selected_device"], action="clear", prep="control")
     cst.clear()
 
 
 @cli.command(short_help="Pause a video.")
 @click.pass_obj
 def pause(settings):
-    cst = setup_cast(settings["device"], action="pause", prep="control")
+    cst = setup_cast(settings["selected_device"], action="pause", prep="control")
     cst.pause()
 
 
 @cli.command(short_help="Resume a video after it has been paused.")
 @click.pass_obj
 def play(settings):
-    cst = setup_cast(settings["device"], action="play", prep="control")
+    cst = setup_cast(settings["selected_device"], action="play", prep="control")
     cst.play()
 
 
 @cli.command("play_toggle", short_help="Toggle between playing and paused state.")
 @click.pass_obj
 def play_toggle(settings):
-    cst = setup_cast(settings["device"], action="play_toggle", prep="control")
+    cst = setup_cast(settings["selected_device"], action="play_toggle", prep="control")
     cst.play_toggle()
 
 
@@ -311,7 +389,7 @@ def play_toggle(settings):
 )
 @click.pass_obj
 def stop(settings, force):
-    cst = setup_cast(settings["device"])
+    cst = setup_cast(settings["selected_device"])
     cst.kill(force=force)
 
 
@@ -319,7 +397,7 @@ def stop(settings, force):
 @click.argument("timedesc", type=CATT_TIME, required=False, default="30", metavar="TIME")
 @click.pass_obj
 def rewind(settings, timedesc):
-    cst = setup_cast(settings["device"], action="rewind", prep="control")
+    cst = setup_cast(settings["selected_device"], action="rewind", prep="control")
     cst.rewind(timedesc)
 
 
@@ -327,7 +405,7 @@ def rewind(settings, timedesc):
 @click.argument("timedesc", type=CATT_TIME, required=False, default="30", metavar="TIME")
 @click.pass_obj
 def ffwd(settings, timedesc):
-    cst = setup_cast(settings["device"], action="ffwd", prep="control")
+    cst = setup_cast(settings["selected_device"], action="ffwd", prep="control")
     cst.ffwd(timedesc)
 
 
@@ -335,14 +413,14 @@ def ffwd(settings, timedesc):
 @click.argument("timedesc", type=CATT_TIME, metavar="TIME")
 @click.pass_obj
 def seek(settings, timedesc):
-    cst = setup_cast(settings["device"], action="seek", prep="control")
+    cst = setup_cast(settings["selected_device"], action="seek", prep="control")
     cst.seek(timedesc)
 
 
 @cli.command(short_help="Skip to end of content.")
 @click.pass_obj
 def skip(settings):
-    cst = setup_cast(settings["device"], action="skip", prep="control")
+    cst = setup_cast(settings["selected_device"], action="skip", prep="control")
     cst.skip()
 
 
@@ -350,7 +428,7 @@ def skip(settings):
 @click.argument("level", type=click.IntRange(0, 100), metavar="LVL")
 @click.pass_obj
 def volume(settings, level):
-    cst = setup_cast(settings["device"])
+    cst = setup_cast(settings["selected_device"])
     cst.volume(level / 100.0)
 
 
@@ -358,7 +436,7 @@ def volume(settings, level):
 @click.argument("delta", type=click.IntRange(1, 100), required=False, default=10, metavar="DELTA")
 @click.pass_obj
 def volumeup(settings, delta):
-    cst = setup_cast(settings["device"])
+    cst = setup_cast(settings["selected_device"])
     cst.volumeup(delta / 100.0)
 
 
@@ -366,14 +444,14 @@ def volumeup(settings, delta):
 @click.argument("delta", type=click.IntRange(1, 100), required=False, default=10, metavar="DELTA")
 @click.pass_obj
 def volumedown(settings, delta):
-    cst = setup_cast(settings["device"])
+    cst = setup_cast(settings["selected_device"])
     cst.volumedown(delta / 100.0)
 
 
 @cli.command(short_help="Show some information about the currently-playing video.")
 @click.pass_obj
 def status(settings):
-    cst = setup_cast(settings["device"], prep="info")
+    cst = setup_cast(settings["selected_device"], prep="info")
     echo_status(cst.cast_info)
 
 
@@ -382,7 +460,7 @@ def status(settings):
 @click.pass_obj
 def info(settings, json_output):
     try:
-        cst = setup_cast(settings["device"], prep="info")
+        cst = setup_cast(settings["selected_device"], prep="info")
     except CastError:
         if json_output:
             info = {}
@@ -403,32 +481,22 @@ def info(settings, json_output):
 def scan(json_output):
     if not json_output:
         click.echo("Scanning Chromecasts...")
-    devices_dict = {
-        d.name: {
-            "host": d.host,
-            "port": d.port,
-            "manufacturer": d.device.manufacturer,
-            "model_name": d.model_name,
-            "uuid": d.uuid,
-            "cast_type": d.cast_type,
-        }
-        for d in get_chromecasts()
-    }
+    devices = get_cast_devices_info()
 
     if json_output:
-        echo_json(devices_dict)
+        echo_json(devices)
     else:
-        if not devices_dict:
+        if not devices:
             raise CastError("No devices found")
-        for device in devices_dict.keys():
-            click.echo("{host} - {device} - {manufacturer} {model_name}".format(device=device, **devices_dict[device]))
+        for device in devices.keys():
+            click.echo("{ip} - {device} - {manufacturer} {model_name}".format(device=device, **devices[device]))
 
 
 @cli.command(short_help="Save the current state of the Chromecast for later use.")
 @click.argument("path", type=click.Path(writable=True), callback=process_path, required=False)
 @click.pass_obj
 def save(settings, path):
-    cst = setup_cast(settings["device"], prep="control")
+    cst = setup_cast(settings["selected_device"], prep="control")
     if not cst.save_capability or cst.is_streaming_local_file:
         raise CliError("Saving state of this kind of content is not supported")
     elif cst.save_capability == "partial":
@@ -453,7 +521,7 @@ def save(settings, path):
 def restore(settings, path):
     if not path and not STATE_PATH.is_file():
         raise CliError("Save file in config dir has not been created")
-    cst = setup_cast(settings["device"])
+    cst = setup_cast(settings["selected_device"])
     state = CastState(path or STATE_PATH, StateMode.READ)
     try:
         data = state.get_data(cst.cc_name if not path else None)
@@ -464,13 +532,13 @@ def restore(settings, path):
 
     echo_status(data["data"])
     click.echo("Restoring...")
-    cst = setup_cast(settings["device"], prep="app", controller=data["controller"])
+    cst = setup_cast(settings["selected_device"], prep="app", controller=data["controller"])
     cst.restore(data["data"])
 
 
-@cli.command("write_config", short_help='Please use "set_default".')
+@cli.command("write_config", short_help='DEPRECATED: Please use "set_default".')
 def write_config():
-    raise CliError('This command is now called "set_default"')
+    raise CliError('DEPRECATED: Please use "set_default"')
 
 
 @cli.command("set_default", short_help="Set the selected device as default.")
@@ -492,7 +560,10 @@ def del_default(settings):
     writeconfig(config)
 
 
-@cli.command("set_alias", short_help="Set an alias name for the selected device.")
+@cli.command(
+    "set_alias",
+    short_help="Set an alias name for the selected device (case-insensitive).",
+)
 @click.argument("name")
 @click.pass_obj
 def set_alias(settings, name):
@@ -525,19 +596,19 @@ def get_alias_from_config(config, device):
 
 
 def get_device_from_settings(settings):
-    device = settings.get("device")
-    if device:
-        devices = get_chromecasts()
-        try:
-            if is_ipaddress(device):
-                next(d for d in devices if d.host == device)
-            else:
-                next(d for d in devices if d.name == device)
-        except StopIteration:
-            raise CliError('Specified device "{}" not found'.format(device))
+    device_desc = settings["selected_device"]
+    if not device_desc or not settings["selected_device_is_from_cli"]:
+        raise CliError("No device specified (must be explicitly specified with -d option)")
+    is_ip = is_ipaddress(device_desc)
+    if is_ip:
+        found = cast_device_ip_exists(device_desc)
     else:
-        raise CliError("No device specified")
-    return device
+        devices = get_cast_devices_info()
+        found = device_desc in devices.keys()
+    if not found:
+        msg = "No device found at {}" if is_ip else 'Specified device "{}" not found'
+        raise CliError(msg.format(device_desc))
+    return device_desc
 
 
 def writeconfig(config):
@@ -561,23 +632,20 @@ def readconfig():
     return config
 
 
-def get_default_map():
+def get_config_as_dict():
     """
     Returns a dictionary of the form:
-        {"option": "value",
+        {"options": {"key": "value"},
          "aliases": {"device1": "device_name"}}
     """
 
     config = readconfig()
-    conf_dict = {section: dict(config.items(section)) for section in config.sections()}
-    default_map = conf_dict["options"]
-    default_map["aliases"] = conf_dict["aliases"]
-    return default_map
+    return {section: dict(config.items(section)) for section in config.sections()}
 
 
 def main():
     try:
-        return cli(obj={}, default_map=get_default_map())
+        return cli(obj=get_config_as_dict())
     except CattUserError as err:
         sys.exit("Error: {}.".format(str(err)))
 
